@@ -32,6 +32,8 @@ DECLARE_MGROUP(BFDD);
 DECLARE_MTYPE(BFDD_CONTROL);
 DECLARE_MTYPE(BFDD_NOTIFICATION);
 
+#define BFDD_SOCK_NAME "%s/bfdd.sock", frr_runstatedir
+
 /* bfd Authentication Type. */
 #define BFD_AUTH_NULL 0
 #define BFD_AUTH_SIMPLE 1
@@ -103,6 +105,8 @@ struct bfd_echo_pkt {
 #define BFD_CBIT 0x08
 #define BFD_ABIT 0x04
 #define BFD_DEMANDBIT 0x02
+#define BFD_MBIT	      0x01
+#define BFD_GETMBIT(flags)    (flags & BFD_MBIT)
 #define BFD_SETDEMANDBIT(flags, val)                                           \
 	{                                                                      \
 		if ((val))                                                     \
@@ -131,7 +135,7 @@ struct bfd_echo_pkt {
 		if ((val))                                                     \
 			flags |= val;                                          \
 	}
-#define BFD_GETCBIT(flags) (flags & BFD_FBIT)
+#define BFD_GETCBIT(flags) (flags & BFD_CBIT)
 #define BFD_ECHO_VERSION 1
 #define BFD_ECHO_PKT_LEN sizeof(struct bfd_echo_pkt)
 
@@ -193,7 +197,7 @@ struct bfd_key {
 	uint16_t mhop;
 	struct in6_addr peer;
 	struct in6_addr local;
-	char ifname[INTERFACE_NAMSIZ];
+	char ifname[IFNAMSIZ];
 	char vrfname[VRF_NAMSIZ];
 } __attribute__((packed));
 
@@ -782,6 +786,7 @@ void bfdd_cli_init(void);
  */
 void bfdd_zclient_init(struct zebra_privs_t *bfdd_priv);
 void bfdd_zclient_stop(void);
+void bfdd_zclient_terminate(void);
 void bfdd_zclient_unregister(vrf_id_t vrf_id);
 void bfdd_zclient_register(vrf_id_t vrf_id);
 void bfdd_sessions_enable_vrf(struct vrf *vrf);
