@@ -15,6 +15,18 @@
 #include "bgpd/bgp_table.h"
 #include "lib/json.h"
 
+struct bgp_addpath_capability {
+	uint16_t afi;
+	uint8_t safi;
+	uint8_t flags;
+};
+
+struct bgp_paths_limit_capability {
+	uint16_t afi;
+	uint8_t safi;
+	uint16_t paths_limit;
+} __attribute__((packed));
+
 #define BGP_ADDPATH_TX_ID_FOR_DEFAULT_ORIGINATE 1
 
 void bgp_addpath_init_bgp_data(struct bgp_addpath_bgp_data *d);
@@ -57,4 +69,6 @@ void bgp_addpath_update_ids(struct bgp *bgp, struct bgp_dest *dest, afi_t afi,
 			    safi_t safi);
 
 void bgp_addpath_type_changed(struct bgp *bgp);
+extern int bgp_addpath_capability_action(enum bgp_addpath_strat addpath_type,
+					 uint8_t paths);
 #endif
