@@ -260,6 +260,8 @@ static struct peer *peer_xfer_conn(struct peer *from_peer)
 		peer->afc_recv[afi][safi] = from_peer->afc_recv[afi][safi];
 		peer->orf_plist[afi][safi] = from_peer->orf_plist[afi][safi];
 		peer->llgr[afi][safi] = from_peer->llgr[afi][safi];
+		peer->addpath_paths_limit[afi][safi] =
+			from_peer->addpath_paths_limit[afi][safi];
 	}
 
 	if (bgp_getsockname(peer) < 0) {
@@ -445,7 +447,7 @@ void bgp_timer_set(struct peer_connection *connection)
 
 		EVENT_OFF(peer->connection->t_pmax_restart);
 		EVENT_OFF(peer->t_refresh_stalepath);
-	/* fallthru */
+		fallthrough;
 	case Clearing:
 		EVENT_OFF(connection->t_start);
 		EVENT_OFF(connection->t_connect);
@@ -562,42 +564,45 @@ void bgp_delayopen_timer(struct event *thread)
 }
 
 /* BGP Peer Down Cause */
-const char *const peer_down_str[] = {"",
-				     "Router ID changed",
-				     "Remote AS changed",
-				     "Local AS change",
-				     "Cluster ID changed",
-				     "Confederation identifier changed",
-				     "Confederation peer changed",
-				     "RR client config change",
-				     "RS client config change",
-				     "Update source change",
-				     "Address family activated",
-				     "Admin. shutdown",
-				     "User reset",
-				     "BGP Notification received",
-				     "BGP Notification send",
-				     "Peer closed the session",
-				     "Neighbor deleted",
-				     "Peer-group add member",
-				     "Peer-group delete member",
-				     "Capability changed",
-				     "Passive config change",
-				     "Multihop config change",
-				     "NSF peer closed the session",
-				     "Intf peering v6only config change",
-				     "BFD down received",
-				     "Interface down",
-				     "Neighbor address lost",
-				     "No path to specified Neighbor",
-				     "Waiting for Peer IPv6 LLA",
-				     "Waiting for VRF to be initialized",
-				     "No AFI/SAFI activated for peer",
-				     "AS Set config change",
-				     "Waiting for peer OPEN",
-				     "Reached received prefix count",
-				     "Socket Error",
-				     "Admin. shutdown (RTT)"};
+const char *const peer_down_str[] = {
+	"",
+	"Router ID changed",
+	"Remote AS changed",
+	"Local AS change",
+	"Cluster ID changed",
+	"Confederation identifier changed",
+	"Confederation peer changed",
+	"RR client config change",
+	"RS client config change",
+	"Update source change",
+	"Address family activated",
+	"Admin. shutdown",
+	"User reset",
+	"BGP Notification received",
+	"BGP Notification send",
+	"Peer closed the session",
+	"Neighbor deleted",
+	"Peer-group add member",
+	"Peer-group delete member",
+	"Capability changed",
+	"Passive config change",
+	"Multihop config change",
+	"NSF peer closed the session",
+	"Intf peering v6only config change",
+	"BFD down received",
+	"Interface down",
+	"Neighbor address lost",
+	"No path to specified Neighbor",
+	"Waiting for Peer IPv6 LLA",
+	"Waiting for VRF to be initialized",
+	"No AFI/SAFI activated for peer",
+	"AS Set config change",
+	"Waiting for peer OPEN",
+	"Reached received prefix count",
+	"Socket Error",
+	"Admin. shutdown (RTT)",
+	"Suppress Fib Turned On or Off",
+};
 
 static void bgp_graceful_restart_timer_off(struct peer_connection *connection,
 					   struct peer *peer)
@@ -682,6 +687,11 @@ static void bgp_set_llgr_stale(struct peer *peer, afi_t afi, safi_t safi)
 						    COMMUNITY_NO_LLGR))
 						continue;
 
+					if (bgp_attr_get_community(pi->attr) &&
+					    community_include(bgp_attr_get_community(pi->attr),
+							      COMMUNITY_LLGR_STALE))
+						continue;
+
 					if (bgp_debug_neighbor_events(peer))
 						zlog_debug(
 							"%pBP Long-lived set stale community (LLGR_STALE) for: %pFX",
@@ -692,8 +702,6 @@ static void bgp_set_llgr_stale(struct peer *peer, afi_t afi, safi_t safi)
 					pi->attr = bgp_attr_intern(&attr);
 					bgp_recalculate_afi_safi_bestpaths(
 						peer->bgp, afi, safi);
-
-					break;
 				}
 		}
 	} else {
@@ -710,6 +718,11 @@ static void bgp_set_llgr_stale(struct peer *peer, afi_t afi, safi_t safi)
 					    COMMUNITY_NO_LLGR))
 					continue;
 
+				if (bgp_attr_get_community(pi->attr) &&
+				    community_include(bgp_attr_get_community(pi->attr),
+						      COMMUNITY_LLGR_STALE))
+					continue;
+
 				if (bgp_debug_neighbor_events(peer))
 					zlog_debug(
 						"%pBP Long-lived set stale community (LLGR_STALE) for: %pFX",
@@ -720,8 +733,6 @@ static void bgp_set_llgr_stale(struct peer *peer, afi_t afi, safi_t safi)
 				pi->attr = bgp_attr_intern(&attr);
 				bgp_recalculate_afi_safi_bestpaths(peer->bgp,
 								   afi, safi);
-
-				break;
 			}
 	}
 }
@@ -1236,7 +1247,7 @@ void bgp_fsm_change_status(struct peer_connection *connection,
 	/* Transition into Clearing or Deleted must /always/ clear all routes..
 	 * (and must do so before actually changing into Deleted..
 	 */
-	if (status >= Clearing) {
+	if (status >= Clearing && (peer->established || peer == bgp->peer_self)) {
 		bgp_clear_route_all(peer);
 
 		/* If no route was queued for the clear-node processing,
@@ -1584,7 +1595,7 @@ bgp_stop_with_error(struct peer_connection *connection)
 
 
 /* something went wrong, send notify and tear down */
-static enum bgp_fsm_state_progress
+enum bgp_fsm_state_progress
 bgp_stop_with_notify(struct peer_connection *connection, uint8_t code,
 		     uint8_t sub_code)
 {
@@ -1774,7 +1785,7 @@ bgp_connect_fail(struct peer_connection *connection)
 {
 	struct peer *peer = connection->peer;
 
-	if (peer_dynamic_neighbor(peer)) {
+	if (peer_dynamic_neighbor_no_nsf(peer)) {
 		if (bgp_debug_neighbor_events(peer))
 			zlog_debug("%s (dynamic neighbor) deleted (%s)",
 				   peer->host, __func__);
@@ -1791,6 +1802,26 @@ bgp_connect_fail(struct peer_connection *connection)
 	return bgp_stop(connection);
 }
 
+/* after connect is called(), getpeername is able to return
+ * port and address on non established streams
+ */
+static void bgp_connect_in_progress_update_connection(struct peer *peer)
+{
+	if (bgp_getsockname(peer) < 0) {
+		if (!peer->su_remote &&
+		    !BGP_CONNECTION_SU_UNSPEC(peer->connection)) {
+			/* if connect initiated, then dest port and dest addresses are well known */
+			peer->su_remote = sockunion_dup(&peer->connection->su);
+			if (sockunion_family(peer->su_remote) == AF_INET)
+				peer->su_remote->sin.sin_port =
+					htons(peer->port);
+			else if (sockunion_family(peer->su_remote) == AF_INET6)
+				peer->su_remote->sin6.sin6_port =
+					htons(peer->port);
+		}
+	}
+}
+
 /* This function is the first starting point of all BGP connection. It
  * try to connect to remote peer with non-blocking IO.
  */
@@ -1887,6 +1918,8 @@ static enum bgp_fsm_state_progress bgp_start(struct peer_connection *connection)
 				 __func__, peer->connection->fd);
 			return BGP_FSM_FAILURE;
 		}
+		bgp_connect_in_progress_update_connection(peer);
+
 		/*
 		 * - when the socket becomes ready, poll() will signify POLLOUT
 		 * - if it fails to connect, poll() will signify POLLHUP
@@ -2384,6 +2417,7 @@ void bgp_fsm_nht_update(struct peer_connection *connection, struct peer *peer,
 		    && (peer->gtsm_hops == BGP_GTSM_HOPS_CONNECTED
 			|| peer->bgp->fast_convergence))
 			BGP_EVENT_ADD(connection, TCP_fatal_error);
+		break;
 	case Clearing:
 	case Deleted:
 	case BGP_STATUS_MAX:
@@ -2716,7 +2750,7 @@ int bgp_gr_lookup_n_update_all_peer(struct bgp *bgp,
 	return BGP_GR_SUCCESS;
 }
 
-int bgp_gr_update_all(struct bgp *bgp, int global_gr_cmd)
+int bgp_gr_update_all(struct bgp *bgp, enum global_gr_command global_gr_cmd)
 {
 	enum global_mode global_new_state = GLOBAL_INVALID;
 	enum global_mode global_old_state = GLOBAL_INVALID;
@@ -2870,7 +2904,8 @@ enum peer_mode bgp_peer_gr_mode_get(struct peer *peer)
 	return peer->peer_gr_present_state;
 }
 
-int bgp_neighbor_graceful_restart(struct peer *peer, int peer_gr_cmd)
+int bgp_neighbor_graceful_restart(struct peer *peer,
+				  enum peer_gr_command peer_gr_cmd)
 {
 	enum peer_mode peer_new_state = PEER_INVALID;
 	enum peer_mode peer_old_state = PEER_INVALID;
@@ -2890,19 +2925,22 @@ int bgp_neighbor_graceful_restart(struct peer *peer, int peer_gr_cmd)
 
 	peer_old_state = bgp_peer_gr_mode_get(peer);
 
-	if (peer_old_state == PEER_INVALID) {
-		zlog_debug("[BGP_GR] peer_old_state == Invalid state !");
+	if (BGP_DEBUG(graceful_restart, GRACEFUL_RESTART))
+		zlog_debug("%s [BGP_GR] peer_old_state: %d", __func__,
+			   peer_old_state);
+
+	if (peer_old_state == PEER_INVALID)
 		return BGP_ERR_GR_OPERATION_FAILED;
-	}
 
 	peer_state = peer->PEER_GR_FSM[peer_old_state][peer_gr_cmd];
 	peer_new_state = peer_state.next_state;
 
-	if (peer_new_state == PEER_INVALID) {
-		zlog_debug(
-			"[BGP_GR] Invalid bgp graceful restart command used !");
+	if (BGP_DEBUG(graceful_restart, GRACEFUL_RESTART))
+		zlog_debug("%s [BGP_GR] peer_new_state: %d", __func__,
+			   peer_new_state);
+
+	if (peer_new_state == PEER_INVALID)
 		return BGP_ERR_GR_INVALID_CMD;
-	}
 
 	if (peer_new_state != peer_old_state) {
 		result = peer_state.action_fun(peer, peer_old_state,
@@ -2929,8 +2967,8 @@ int bgp_neighbor_graceful_restart(struct peer *peer, int peer_gr_cmd)
 	return result;
 }
 
-unsigned int bgp_peer_gr_action(struct peer *peer, int old_peer_state,
-				int new_peer_state)
+unsigned int bgp_peer_gr_action(struct peer *peer, enum peer_mode old_peer_state,
+				enum peer_mode new_peer_state)
 {
 	if (BGP_DEBUG(graceful_restart, GRACEFUL_RESTART))
 		zlog_debug(
@@ -2938,7 +2976,7 @@ unsigned int bgp_peer_gr_action(struct peer *peer, int old_peer_state,
 			__func__, print_peer_gr_mode(old_peer_state),
 			print_peer_gr_mode(new_peer_state));
 
-	int bgp_gr_global_mode = GLOBAL_INVALID;
+	enum global_mode bgp_gr_global_mode = GLOBAL_INVALID;
 	unsigned int ret = BGP_GR_FAILURE;
 
 	if (old_peer_state == new_peer_state) {
@@ -2970,10 +3008,10 @@ unsigned int bgp_peer_gr_action(struct peer *peer, int old_peer_state,
 
 		BGP_PEER_GR_GLOBAL_INHERIT_UNSET(peer);
 
-		if (new_peer_state == bgp_gr_global_mode) {
-			/*This is incremental updates i.e no tear down
-			 *of the existing session
-			 *as the peer is already working in the same mode.
+		if ((int)new_peer_state == (int)bgp_gr_global_mode) {
+			/* This is incremental updates i.e no tear down
+			 * of the existing session
+			 * as the peer is already working in the same mode.
 			 */
 			ret = BGP_GR_SUCCESS;
 		} else {
@@ -3000,8 +3038,7 @@ unsigned int bgp_peer_gr_action(struct peer *peer, int old_peer_state,
 
 		BGP_PEER_GR_GLOBAL_INHERIT_SET(peer);
 
-		if (old_peer_state == bgp_gr_global_mode) {
-
+		if ((int)old_peer_state == (int)bgp_gr_global_mode) {
 			/* This is incremental updates
 			 *i.e no tear down of the existing session
 			 *as the peer is already working in the same mode.
