@@ -11,6 +11,7 @@
 
 DECLARE_MGROUP(BGPD);
 DECLARE_MTYPE(BGP);
+DECLARE_MTYPE(BGP_NAME);
 DECLARE_MTYPE(BGP_LISTENER);
 DECLARE_MTYPE(BGP_PEER);
 DECLARE_MTYPE(BGP_PEER_CONNECTION);
@@ -37,6 +38,7 @@ DECLARE_MTYPE(BGP_ROUTE_EXTRA);
 DECLARE_MTYPE(BGP_ROUTE_EXTRA_EVPN);
 DECLARE_MTYPE(BGP_ROUTE_EXTRA_FS);
 DECLARE_MTYPE(BGP_ROUTE_EXTRA_VRFLEAK);
+DECLARE_MTYPE(BGP_ROUTE_EXTRA_VNC);
 DECLARE_MTYPE(BGP_CONN);
 DECLARE_MTYPE(BGP_STATIC);
 DECLARE_MTYPE(BGP_ADVERTISE_ATTR);
@@ -85,6 +87,7 @@ DECLARE_MTYPE(PEER_UPDATE_SOURCE);
 DECLARE_MTYPE(PEER_CONF_IF);
 DECLARE_MTYPE(BGP_DAMP_INFO);
 DECLARE_MTYPE(BGP_DAMP_ARRAY);
+DECLARE_MTYPE(BGP_DAMP_REUSELIST);
 DECLARE_MTYPE(BGP_REGEXP);
 DECLARE_MTYPE(BGP_AGGREGATE);
 DECLARE_MTYPE(BGP_ADDR);
@@ -95,6 +98,8 @@ DECLARE_MTYPE(BGP_FILTER_NAME);
 DECLARE_MTYPE(BGP_DUMP_STR);
 DECLARE_MTYPE(ENCAP_TLV);
 
+DECLARE_MTYPE(BGP_LABELS);
+
 DECLARE_MTYPE(BGP_TEA_OPTIONS);
 DECLARE_MTYPE(BGP_TEA_OPTIONS_VALUE);
 
