@@ -146,7 +146,7 @@ struct yang_translator *yang_translator_load(const char *path)
 	 */
 	assert(dnode);
 
-	family = yang_dnode_get_string(dnode, "./family");
+	family = yang_dnode_get_string(dnode, "family");
 	translator = yang_translator_find(family);
 	if (translator != NULL) {
 		flog_warn(EC_LIB_YANG_TRANSLATOR_LOAD,
@@ -182,7 +182,7 @@ struct yang_translator *yang_translator_load(const char *path)
 		tmodule =
 			XCALLOC(MTYPE_YANG_TRANSLATOR_MODULE, sizeof(*tmodule));
 
-		module_name = yang_dnode_get_string(set->dnodes[i], "./name");
+		module_name = yang_dnode_get_string(set->dnodes[i], "name");
 		tmodule->module = ly_ctx_load_module(translator->ly_ctx,
 						     module_name, NULL, NULL);
 		if (!tmodule->module) {
@@ -233,7 +233,7 @@ struct yang_translator *yang_translator_load(const char *path)
 		const struct lysc_node *snode_custom, *snode_native;
 
 		xpath_custom =
-			yang_dnode_get_string(set->dnodes[i], "./custom");
+			yang_dnode_get_string(set->dnodes[i], "custom");
 
 		snode_custom =
 			yang_find_snode(translator->ly_ctx, xpath_custom, 0);
@@ -246,7 +246,7 @@ struct yang_translator *yang_translator_load(const char *path)
 		}
 
 		xpath_native =
-			yang_dnode_get_string(set->dnodes[i], "./native");
+			yang_dnode_get_string(set->dnodes[i], "native");
 		snode_native = yang_find_snode(ly_native_ctx, xpath_native, 0);
 		if (!snode_native) {
 			flog_warn(EC_LIB_YANG_TRANSLATOR_LOAD,
