@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2003 Yasuhiro Ohara
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -27,7 +12,7 @@
 #include "vty.h"
 #include "command.h"
 #include "memory.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "checksum.h"
 #include "frrstr.h"
 
@@ -78,10 +63,10 @@ struct ospf6 *ospf6_get_by_lsdb(struct ospf6_lsa *lsa)
 static int ospf6_unknown_lsa_show(struct vty *vty, struct ospf6_lsa *lsa,
 				  json_object *json_obj, bool use_json)
 {
-	uint8_t *start, *end, *current;
+	char *start, *end, *current;
 
-	start = (uint8_t *)lsa->header + sizeof(struct ospf6_lsa_header);
-	end = (uint8_t *)lsa->header + ntohs(lsa->header->length);
+	start = ospf6_lsa_header_end(lsa->header);
+	end = ospf6_lsa_end(lsa->header);
 
 	if (use_json) {
 		json_object_string_add(json_obj, "lsaType", "unknown");
@@ -216,10 +201,10 @@ int ospf6_lsa_is_differ(struct ospf6_lsa *lsa1, struct ospf6_lsa *lsa2)
 		return 1;
 
 	/* compare body */
-	if (ntohs(lsa1->header->length) != ntohs(lsa2->header->length))
+	if (ospf6_lsa_size(lsa1->header) != ospf6_lsa_size(lsa2->header))
 		return 1;
 
-	len = ntohs(lsa1->header->length) - sizeof(struct ospf6_lsa_header);
+	len = ospf6_lsa_size(lsa1->header) - sizeof(struct ospf6_lsa_header);
 	return memcmp(lsa1->header + 1, lsa2->header + 1, len);
 }
 
@@ -229,7 +214,7 @@ int ospf6_lsa_is_changed(struct ospf6_lsa *lsa1, struct ospf6_lsa *lsa2)
 
 	if (OSPF6_LSA_IS_MAXAGE(lsa1) ^ OSPF6_LSA_IS_MAXAGE(lsa2))
 		return 1;
-	if (ntohs(lsa1->header->length) != ntohs(lsa2->header->length))
+	if (ospf6_lsa_size(lsa1->header) != ospf6_lsa_size(lsa2->header))
 		return 1;
 	/* Going beyond LSA headers to compare the payload only makes sense,
 	 * when both LSAs aren't header-only. */
@@ -243,14 +228,14 @@ int ospf6_lsa_is_changed(struct ospf6_lsa *lsa1, struct ospf6_lsa *lsa2)
 	if (CHECK_FLAG(lsa1->flag, OSPF6_LSA_HEADERONLY))
 		return 0;
 
-	length = OSPF6_LSA_SIZE(lsa1->header) - sizeof(struct ospf6_lsa_header);
+	length = ospf6_lsa_size(lsa1->header) - sizeof(struct ospf6_lsa_header);
 	/* Once upper layer verifies LSAs received, length underrun should
 	 * become a warning. */
 	if (length <= 0)
 		return 0;
 
-	return memcmp(OSPF6_LSA_HEADER_END(lsa1->header),
-		      OSPF6_LSA_HEADER_END(lsa2->header), length);
+	return memcmp(ospf6_lsa_header_end(lsa1->header),
+		      ospf6_lsa_header_end(lsa2->header), length);
 }
 
 /* ospf6 age functions */
@@ -317,8 +302,8 @@ void ospf6_lsa_premature_aging(struct ospf6_lsa *lsa)
 	if (IS_OSPF6_DEBUG_LSA_TYPE(lsa->header->type))
 		zlog_debug("LSA: Premature aging: %s", lsa->name);
 
-	THREAD_OFF(lsa->expire);
-	THREAD_OFF(lsa->refresh);
+	EVENT_OFF(lsa->expire);
+	EVENT_OFF(lsa->refresh);
 
 	/*
 	 * We clear the LSA from the neighbor retx lists now because it
@@ -348,7 +333,7 @@ void ospf6_lsa_premature_aging(struct ospf6_lsa *lsa)
 	ospf6_flood_clear(lsa);
 
 	lsa->header->age = htons(OSPF_LSA_MAXAGE);
-	thread_execute(master, ospf6_lsa_expire, lsa, 0);
+	event_execute(master, ospf6_lsa_expire, lsa, 0, NULL);
 }
 
 /* check which is more recent. if a is more recent, return -1;
@@ -563,7 +548,7 @@ void ospf6_lsa_show_dump(struct vty *vty, struct ospf6_lsa *lsa,
 	json_object *json = NULL;
 
 	start = (uint8_t *)lsa->header;
-	end = (uint8_t *)lsa->header + ntohs(lsa->header->length);
+	end = (uint8_t *)ospf6_lsa_end(lsa->header);
 
 	if (use_json) {
 		json = json_object_new_object();
@@ -628,7 +613,7 @@ void ospf6_lsa_show_internal(struct vty *vty, struct ospf6_lsa *lsa,
 		json_object_int_add(json_obj, "checksum",
 				    ntohs(lsa->header->checksum));
 		json_object_int_add(json_obj, "length",
-				    ntohs(lsa->header->length));
+				    ospf6_lsa_size(lsa->header));
 		json_object_int_add(json_obj, "flag", lsa->flag);
 		json_object_int_add(json_obj, "lock", lsa->lock);
 		json_object_int_add(json_obj, "reTxCount", lsa->retrans_count);
@@ -645,7 +630,7 @@ void ospf6_lsa_show_internal(struct vty *vty, struct ospf6_lsa *lsa,
 			(unsigned long)ntohl(lsa->header->seqnum));
 		vty_out(vty, "CheckSum: %#06hx Length: %hu\n",
 			ntohs(lsa->header->checksum),
-			ntohs(lsa->header->length));
+			ospf6_lsa_size(lsa->header));
 		vty_out(vty, "Flag: %x \n", lsa->flag);
 		vty_out(vty, "Lock: %d \n", lsa->lock);
 		vty_out(vty, "ReTx Count: %d\n", lsa->retrans_count);
@@ -735,7 +720,7 @@ struct ospf6_lsa *ospf6_lsa_create(struct ospf6_lsa_header *header)
 	uint16_t lsa_size = 0;
 
 	/* size of the entire LSA */
-	lsa_size = ntohs(header->length); /* XXX vulnerable */
+	lsa_size = ospf6_lsa_size(header); /* XXX vulnerable */
 
 	lsa = ospf6_lsa_alloc(lsa_size);
 
@@ -775,8 +760,8 @@ void ospf6_lsa_delete(struct ospf6_lsa *lsa)
 	assert(lsa->lock == 0);
 
 	/* cancel threads */
-	THREAD_OFF(lsa->expire);
-	THREAD_OFF(lsa->refresh);
+	EVENT_OFF(lsa->expire);
+	EVENT_OFF(lsa->refresh);
 
 	/* do free */
 	XFREE(MTYPE_OSPF6_LSA_HEADER, lsa->header);
@@ -812,33 +797,33 @@ struct ospf6_lsa *ospf6_lsa_lock(struct ospf6_lsa *lsa)
 }
 
 /* decrement reference counter of struct ospf6_lsa */
-struct ospf6_lsa *ospf6_lsa_unlock(struct ospf6_lsa *lsa)
+void ospf6_lsa_unlock(struct ospf6_lsa **lsa)
 {
 	/* decrement reference counter */
-	assert(lsa->lock > 0);
-	lsa->lock--;
+	assert((*lsa)->lock > 0);
+	(*lsa)->lock--;
 
-	if (lsa->lock != 0)
-		return lsa;
+	if ((*lsa)->lock != 0)
+		return;
 
-	ospf6_lsa_delete(lsa);
-	return NULL;
+	ospf6_lsa_delete(*lsa);
+	*lsa = NULL;
 }
 
 
 /* ospf6 lsa expiry */
-void ospf6_lsa_expire(struct thread *thread)
+void ospf6_lsa_expire(struct event *thread)
 {
 	struct ospf6_lsa *lsa;
 	struct ospf6 *ospf6;
 
-	lsa = (struct ospf6_lsa *)THREAD_ARG(thread);
+	lsa = (struct ospf6_lsa *)EVENT_ARG(thread);
 
 	assert(lsa && lsa->header);
 	assert(OSPF6_LSA_IS_MAXAGE(lsa));
 	assert(!lsa->refresh);
 
-	lsa->expire = (struct thread *)NULL;
+	lsa->expire = (struct event *)NULL;
 
 	if (IS_OSPF6_DEBUG_LSA_TYPE(lsa->header->type)) {
 		zlog_debug("LSA Expire:");
@@ -860,15 +845,15 @@ void ospf6_lsa_expire(struct thread *thread)
 	ospf6_maxage_remove(ospf6);
 }
 
-void ospf6_lsa_refresh(struct thread *thread)
+void ospf6_lsa_refresh(struct event *thread)
 {
 	struct ospf6_lsa *old, *self, *new;
 	struct ospf6_lsdb *lsdb_self;
 
-	old = (struct ospf6_lsa *)THREAD_ARG(thread);
+	old = (struct ospf6_lsa *)EVENT_ARG(thread);
 	assert(old && old->header);
 
-	old->refresh = (struct thread *)NULL;
+	old->refresh = (struct event *)NULL;
 
 	lsdb_self = ospf6_get_scoped_lsdb_self(old);
 	self = ospf6_lsdb_lookup(old->header->type, old->header->id,
@@ -890,8 +875,8 @@ void ospf6_lsa_refresh(struct thread *thread)
 
 	new = ospf6_lsa_create(self->header);
 	new->lsdb = old->lsdb;
-	thread_add_timer(master, ospf6_lsa_refresh, new, OSPF_LS_REFRESH_TIME,
-			 &new->refresh);
+	event_add_timer(master, ospf6_lsa_refresh, new, OSPF_LS_REFRESH_TIME,
+			&new->refresh);
 
 	/* store it in the LSDB for self-originated LSAs */
 	ospf6_lsdb_add(ospf6_lsa_copy(new), lsdb_self);
@@ -962,7 +947,7 @@ unsigned short ospf6_lsa_checksum(struct ospf6_lsa_header *lsa_header)
 		buffer - (uint8_t *)&lsa_header->age; /* should be 2 */
 
 	/* Skip the AGE field */
-	uint16_t len = ntohs(lsa_header->length) - type_offset;
+	uint16_t len = ospf6_lsa_size(lsa_header) - type_offset;
 
 	/* Checksum offset starts from "type" field, not the beginning of the
 	   lsa_header struct. The offset is 14, rather than 16. */
@@ -978,7 +963,7 @@ int ospf6_lsa_checksum_valid(struct ospf6_lsa_header *lsa_header)
 		buffer - (uint8_t *)&lsa_header->age; /* should be 2 */
 
 	/* Skip the AGE field */
-	uint16_t len = ntohs(lsa_header->length) - type_offset;
+	uint16_t len = ospf6_lsa_size(lsa_header) - type_offset;
 
 	return (fletcher_checksum(buffer, len, FLETCHER_CHECKSUM_VALIDATE)
 		== 0);
