@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * IP MSDP socket management
  * Copyright (C) 2016 Cumulus Networks, Inc.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -22,19 +9,22 @@
 #include <lib/log.h>
 #include <lib/network.h>
 #include <lib/sockunion.h>
-#include <lib/thread.h>
+#include "frrevent.h"
 #include <lib/vty.h>
 #include <lib/if.h>
 #include <lib/vrf.h>
 #include <lib/lib_errors.h>
 
 #include "pimd.h"
+#include "pim_instance.h"
 #include "pim_sock.h"
 #include "pim_errors.h"
 
 #include "pim_msdp.h"
 #include "pim_msdp_socket.h"
 
+#include "sockopt.h"
+
 /* increase socket send buffer size */
 static void pim_msdp_update_sock_send_buffer_size(int fd)
 {
@@ -43,8 +33,8 @@ static void pim_msdp_update_sock_send_buffer_size(int fd)
 	socklen_t optlen = sizeof(optval);
 
 	if (getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &optval, &optlen) < 0) {
-		flog_err_sys(LIB_ERR_SOCKET,
-			     "getsockopt of SO_SNDBUF failed %s\n",
+		flog_err_sys(EC_LIB_SOCKET,
+			     "getsockopt of SO_SNDBUF failed %s",
 			     safe_strerror(errno));
 		return;
 	}
@@ -52,42 +42,41 @@ static void pim_msdp_update_sock_send_buffer_size(int fd)
 	if (optval < size) {
 		if (setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &size, sizeof(size))
 		    < 0) {
-			flog_err_sys(LIB_ERR_SOCKET,
-				     "Couldn't increase send buffer: %s\n",
+			flog_err_sys(EC_LIB_SOCKET,
+				     "Couldn't increase send buffer: %s",
 				     safe_strerror(errno));
 		}
 	}
 }
 
 /* passive peer socket accept */
-static int pim_msdp_sock_accept(struct thread *thread)
+static void pim_msdp_sock_accept(struct event *thread)
 {
 	union sockunion su;
-	struct pim_instance *pim = THREAD_ARG(thread);
+	struct pim_instance *pim = EVENT_ARG(thread);
 	int accept_sock;
 	int msdp_sock;
 	struct pim_msdp_peer *mp;
-	char buf[SU_ADDRSTRLEN];
 
 	sockunion_init(&su);
 
 	/* re-register accept thread */
-	accept_sock = THREAD_FD(thread);
+	accept_sock = EVENT_FD(thread);
 	if (accept_sock < 0) {
-		flog_err(LIB_ERR_DEVELOPMENT,
-			  "accept_sock is negative value %d", accept_sock);
-		return -1;
+		flog_err(EC_LIB_DEVELOPMENT, "accept_sock is negative value %d",
+			 accept_sock);
+		return;
 	}
 	pim->msdp.listener.thread = NULL;
-	thread_add_read(master, pim_msdp_sock_accept, pim, accept_sock,
-			&pim->msdp.listener.thread);
+	event_add_read(router->master, pim_msdp_sock_accept, pim, accept_sock,
+		       &pim->msdp.listener.thread);
 
 	/* accept client connection. */
 	msdp_sock = sockunion_accept(accept_sock, &su);
 	if (msdp_sock < 0) {
-		flog_err_sys(LIB_ERR_SOCKET, "pim_msdp_sock_accept failed (%s)",
+		flog_err_sys(EC_LIB_SOCKET, "pim_msdp_sock_accept failed (%s)",
 			     safe_strerror(errno));
-		return -1;
+		return;
 	}
 
 	/* see if have peer config for this */
@@ -95,12 +84,11 @@ static int pim_msdp_sock_accept(struct thread *thread)
 	if (!mp || !PIM_MSDP_PEER_IS_LISTENER(mp)) {
 		++pim->msdp.rejected_accepts;
 		if (PIM_DEBUG_MSDP_EVENTS) {
-			flog_err(PIM_ERR_MSDP_PACKET,
-				  "msdp peer connection refused from %s",
-				  sockunion2str(&su, buf, SU_ADDRSTRLEN));
+			flog_err(EC_PIM_MSDP_PACKET,
+				 "msdp peer connection refused from %pSU", &su);
 		}
 		close(msdp_sock);
-		return -1;
+		return;
 	}
 
 	if (PIM_DEBUG_MSDP_INTERNAL) {
@@ -113,8 +101,8 @@ static int pim_msdp_sock_accept(struct thread *thread)
 	if (mp->fd >= 0) {
 		if (PIM_DEBUG_MSDP_EVENTS) {
 			zlog_notice(
-				"msdp peer new connection from %s stop old connection",
-				sockunion2str(&su, buf, SU_ADDRSTRLEN));
+				"msdp peer new connection from %pSU stop old connection",
+				&su);
 		}
 		pim_msdp_peer_stop_tcp_conn(mp, true /* chg_state */);
 	}
@@ -122,7 +110,6 @@ static int pim_msdp_sock_accept(struct thread *thread)
 	set_nonblocking(mp->fd);
 	pim_msdp_update_sock_send_buffer_size(mp->fd);
 	pim_msdp_peer_established(mp);
-	return 0;
 }
 
 /* global listener for the MSDP well know TCP port */
@@ -141,8 +128,7 @@ int pim_msdp_sock_listen(struct pim_instance *pim)
 
 	sock = socket(AF_INET, SOCK_STREAM, 0);
 	if (sock < 0) {
-		flog_err_sys(LIB_ERR_SOCKET, "socket: %s",
-			     safe_strerror(errno));
+		flog_err_sys(EC_LIB_SOCKET, "socket: %s", safe_strerror(errno));
 		return sock;
 	}
 
@@ -157,32 +143,32 @@ int pim_msdp_sock_listen(struct pim_instance *pim)
 	sockopt_reuseaddr(sock);
 	sockopt_reuseport(sock);
 
-	if (pim->vrf_id != VRF_DEFAULT) {
+	if (pim->vrf->vrf_id != VRF_DEFAULT) {
 		struct interface *ifp =
-			if_lookup_by_name(pim->vrf->name, pim->vrf_id);
+			if_lookup_by_name(pim->vrf->name, pim->vrf->vrf_id);
 		if (!ifp) {
-			flog_err(LIB_ERR_INTERFACE,
-				  "%s: Unable to lookup vrf interface: %s",
-				  __PRETTY_FUNCTION__, pim->vrf->name);
+			flog_err(EC_LIB_INTERFACE,
+				 "%s: Unable to lookup vrf interface: %s",
+				 __func__, pim->vrf->name);
 			close(sock);
 			return -1;
 		}
 		if (pim_socket_bind(sock, ifp)) {
-			flog_err_sys(LIB_ERR_SOCKET,
+			flog_err_sys(EC_LIB_SOCKET,
 				     "%s: Unable to bind to socket: %s",
-				     __PRETTY_FUNCTION__, safe_strerror(errno));
+				     __func__, safe_strerror(errno));
 			close(sock);
 			return -1;
 		}
 	}
 
-	frr_elevate_privs(&pimd_privs) {
+	frr_with_privs(&pimd_privs) {
 		/* bind to well known TCP port */
 		rc = bind(sock, (struct sockaddr *)&sin, socklen);
 	}
 
 	if (rc < 0) {
-		flog_err_sys(LIB_ERR_SOCKET,
+		flog_err_sys(EC_LIB_SOCKET,
 			     "pim_msdp_socket bind to port %d: %s",
 			     ntohs(sin.sin_port), safe_strerror(errno));
 		close(sock);
@@ -191,18 +177,23 @@ int pim_msdp_sock_listen(struct pim_instance *pim)
 
 	rc = listen(sock, 3 /* backlog */);
 	if (rc < 0) {
-		flog_err_sys(LIB_ERR_SOCKET, "pim_msdp_socket listen: %s",
+		flog_err_sys(EC_LIB_SOCKET, "pim_msdp_socket listen: %s",
 			     safe_strerror(errno));
 		close(sock);
 		return rc;
 	}
 
+	/* Set socket DSCP byte */
+	if (setsockopt_ipv4_tos(sock, IPTOS_PREC_INTERNETCONTROL)) {
+		zlog_warn("can't set sockopt IP_TOS to MSDP socket %d: %s",
+				sock, safe_strerror(errno));
+	}
+
 	/* add accept thread */
 	listener->fd = sock;
 	memcpy(&listener->su, &sin, socklen);
-	listener->thread = NULL;
-	thread_add_read(pim->msdp.master, pim_msdp_sock_accept, pim, sock,
-			&listener->thread);
+	event_add_read(pim->msdp.master, pim_msdp_sock_accept, pim, sock,
+		       &listener->thread);
 
 	pim->msdp.flags |= PIM_MSDPF_LISTENER;
 	return 0;
@@ -232,25 +223,25 @@ int pim_msdp_sock_connect(struct pim_msdp_peer *mp)
 	/* Make socket for the peer. */
 	mp->fd = sockunion_socket(&mp->su_peer);
 	if (mp->fd < 0) {
-		flog_err_sys(LIB_ERR_SOCKET,
+		flog_err_sys(EC_LIB_SOCKET,
 			     "pim_msdp_socket socket failure: %s",
 			     safe_strerror(errno));
 		return -1;
 	}
 
-	if (mp->pim->vrf_id != VRF_DEFAULT) {
-		struct interface *ifp =
-			if_lookup_by_name(mp->pim->vrf->name, mp->pim->vrf_id);
+	if (mp->pim->vrf->vrf_id != VRF_DEFAULT) {
+		struct interface *ifp = if_lookup_by_name(mp->pim->vrf->name,
+							  mp->pim->vrf->vrf_id);
 		if (!ifp) {
-			flog_err(LIB_ERR_INTERFACE,
-				  "%s: Unable to lookup vrf interface: %s",
-				  __PRETTY_FUNCTION__, mp->pim->vrf->name);
+			flog_err(EC_LIB_INTERFACE,
+				 "%s: Unable to lookup vrf interface: %s",
+				 __func__, mp->pim->vrf->name);
 			return -1;
 		}
 		if (pim_socket_bind(mp->fd, ifp)) {
-			flog_err_sys(LIB_ERR_SOCKET,
+			flog_err_sys(EC_LIB_SOCKET,
 				     "%s: Unable to bind to socket: %s",
-				     __PRETTY_FUNCTION__, safe_strerror(errno));
+				     __func__, safe_strerror(errno));
 			close(mp->fd);
 			mp->fd = -1;
 			return -1;
@@ -267,7 +258,7 @@ int pim_msdp_sock_connect(struct pim_msdp_peer *mp)
 	/* source bind */
 	rc = sockunion_bind(mp->fd, &mp->su_local, 0, &mp->su_local);
 	if (rc < 0) {
-		flog_err_sys(LIB_ERR_SOCKET,
+		flog_err_sys(EC_LIB_SOCKET,
 			     "pim_msdp_socket connect bind failure: %s",
 			     safe_strerror(errno));
 		close(mp->fd);
@@ -275,6 +266,12 @@ int pim_msdp_sock_connect(struct pim_msdp_peer *mp)
 		return rc;
 	}
 
+	/* Set socket DSCP byte */
+	if (setsockopt_ipv4_tos(mp->fd, IPTOS_PREC_INTERNETCONTROL)) {
+		zlog_warn("can't set sockopt IP_TOS to MSDP socket %d: %s",
+				mp->fd, safe_strerror(errno));
+	}
+
 	/* Connect to the remote mp. */
 	return (sockunion_connect(mp->fd, &mp->su_peer,
 				  htons(PIM_MSDP_TCP_PORT), 0));
