@@ -743,14 +743,13 @@ static int hello_send(struct interface *ifp, uint16_t holdtime)
 	pim_ifp = ifp->info;
 
 	if (PIM_DEBUG_PIM_HELLO)
-		zlog_debug(
-			"%s: to %pPA on %s: holdt=%u prop_d=%u overr_i=%u dis_join_supp=%d dr_prio=%u gen_id=%08x addrs=%d",
-			__func__, &qpim_all_pim_routers_addr, ifp->name,
-			holdtime, pim_ifp->pim_propagation_delay_msec,
-			pim_ifp->pim_override_interval_msec,
-			pim_ifp->pim_can_disable_join_suppression,
-			pim_ifp->pim_dr_priority, pim_ifp->pim_generation_id,
-			listcount(ifp->connected));
+		zlog_debug("%s: to %pPA on %s: holdt=%u prop_d=%u overr_i=%u dis_join_supp=%d dr_prio=%u gen_id=%08x addrs=%zu",
+			   __func__, &qpim_all_pim_routers_addr, ifp->name,
+			   holdtime, pim_ifp->pim_propagation_delay_msec,
+			   pim_ifp->pim_override_interval_msec,
+			   pim_ifp->pim_can_disable_join_suppression,
+			   pim_ifp->pim_dr_priority, pim_ifp->pim_generation_id,
+			   if_connected_count(ifp->connected));
 
 	pim_tlv_size = pim_hello_build_tlv(
 		ifp, pim_msg + PIM_PIM_MIN_LEN,
