@@ -25,6 +25,7 @@ from collections import OrderedDict
 from ipaddress import IPv6Address, ip_network
 from pprint import pformat
 
+
 # Python 3
 def iteritems(d):
     return iter(d.items())
@@ -219,6 +220,23 @@ def get_normalized_mac_ip_line(line):
     return line
 
 
+def get_normalized_interface_vrf(line):
+    """
+    If 'interface <int_name> vrf <vrf_name>' is present in file,
+    we need to remove the explicit "vrf <vrf_name>"
+    so that the context information is created
+    correctly and configurations are matched appropriately.
+    """
+
+    intf_vrf = re.search("interface (\S+) vrf (\S+)", line)
+    if intf_vrf:
+        old_line = "vrf %s" % intf_vrf.group(2)
+        new_line = line.replace(old_line, "").strip()
+        return new_line
+
+    return line
+
+
 # This dictionary contains a tree of all commands that we know start a
 # new multi-line context. All other commands are treated either as
 # commands inside a multi-line context or as single-line contexts. This
@@ -294,6 +312,10 @@ class Config(object):
             # Compress duplicate whitespaces
             line = " ".join(line.split())
 
+            # Remove 'vrf <vrf_name>' from 'interface <x> vrf <vrf_name>'
+            if line.startswith("interface ") and "vrf" in line:
+                line = get_normalized_interface_vrf(line)
+
             if ":" in line:
                 line = get_normalized_mac_ip_line(line)
 
@@ -362,7 +384,7 @@ class Config(object):
         """
         Return the parsed context as strings for display, log etc.
         """
-        for (_, ctx) in sorted(iteritems(self.contexts)):
+        for _, ctx in sorted(iteritems(self.contexts)):
             print(str(ctx))
 
     def save_contexts(self, key, lines):
@@ -546,7 +568,6 @@ class Config(object):
         cur_ctx_lines = []
 
         for line in self.lines:
-
             if not line:
                 continue
 
@@ -647,7 +668,7 @@ def lines_to_config(ctx_keys, line, delete):
             ctx_keys = []
 
     if line:
-        for (i, ctx_key) in enumerate(ctx_keys):
+        for i, ctx_key in enumerate(ctx_keys):
             cmd.append(" " * i + ctx_key)
 
         line = line.lstrip()
@@ -725,7 +746,7 @@ def get_normalized_ipv6_line(line):
 
 
 def line_exist(lines, target_ctx_keys, target_line, exact_match=True):
-    for (ctx_keys, line) in lines:
+    for ctx_keys, line in lines:
         if ctx_keys == target_ctx_keys:
             if exact_match:
                 if line == target_line:
@@ -744,7 +765,7 @@ def bgp_delete_inst_move_line(lines_to_del):
     bgp_defult_inst = False
     bgp_vrf_inst = False
 
-    for (ctx_keys, line) in lines_to_del:
+    for ctx_keys, line in lines_to_del:
         # Find bgp default inst
         if (
             ctx_keys[0].startswith("router bgp")
@@ -757,7 +778,7 @@ def bgp_delete_inst_move_line(lines_to_del):
             bgp_vrf_inst = True
 
     if bgp_defult_inst and bgp_vrf_inst:
-        for (ctx_keys, line) in lines_to_del:
+        for ctx_keys, line in lines_to_del:
             # move bgp default inst to end
             if (
                 ctx_keys[0].startswith("router bgp")
@@ -853,7 +874,6 @@ def bgp_delete_nbr_remote_as_line(lines_to_add):
 
 
 def bgp_remove_neighbor_cfg(lines_to_del, del_nbr_dict):
-
     # This method handles deletion of bgp neighbor configs,
     # if there is neighbor to peer-group cmd is in delete list.
     # As 'no neighbor .* peer-group' deletes the neighbor,
@@ -861,7 +881,7 @@ def bgp_remove_neighbor_cfg(lines_to_del, del_nbr_dict):
     # in error.
     lines_to_del_to_del = []
 
-    for (ctx_keys, line) in lines_to_del:
+    for ctx_keys, line in lines_to_del:
         if (
             ctx_keys[0].startswith("router bgp")
             and line
@@ -876,7 +896,7 @@ def bgp_remove_neighbor_cfg(lines_to_del, del_nbr_dict):
                         if re_nb:
                             lines_to_del_to_del.append((ctx_keys, line))
 
-    for (ctx_keys, line) in lines_to_del_to_del:
+    for ctx_keys, line in lines_to_del_to_del:
         lines_to_del.remove((ctx_keys, line))
 
 
@@ -942,7 +962,7 @@ def bgp_delete_move_lines(lines_to_add, lines_to_del):
     #   "router bgp 200  no neighbor uplink1 interface remote-as internal"
     #   "router bgp 200  no neighbor underlay peer-group"
 
-    for (ctx_keys, line) in lines_to_del:
+    for ctx_keys, line in lines_to_del:
         if (
             ctx_keys[0].startswith("router bgp")
             and line
@@ -993,19 +1013,20 @@ def bgp_delete_move_lines(lines_to_add, lines_to_del):
                 del_dict[ctx_keys[0]][re_pg.group(1)] = list()
                 found_pg_del_cmd = True
 
+    # move neighbor remote-as lines at the end
+    for ctx_keys, line in lines_to_del_to_app:
+        lines_to_del.remove((ctx_keys, line))
+        lines_to_del.append((ctx_keys, line))
+
     if found_pg_del_cmd == False:
         bgp_delete_inst_move_line(lines_to_del)
         if del_nbr_dict:
             bgp_remove_neighbor_cfg(lines_to_del, del_nbr_dict)
         return (lines_to_add, lines_to_del)
 
-    for (ctx_keys, line) in lines_to_del_to_app:
-        lines_to_del.remove((ctx_keys, line))
-        lines_to_del.append((ctx_keys, line))
-
     # {'router bgp 65001': {'PG': ['10.1.1.2'], 'PG1': ['10.1.1.21']},
     #  'router bgp 65001 vrf vrf1': {'PG': ['10.1.1.2'], 'PG1': ['10.1.1.21']}}
-    for (ctx_keys, line) in lines_to_del:
+    for ctx_keys, line in lines_to_del:
         if (
             ctx_keys[0].startswith("router bgp")
             and line
@@ -1023,7 +1044,7 @@ def bgp_delete_move_lines(lines_to_add, lines_to_del):
                         del_dict[ctx_keys[0]][pg_key].append(re_nbr_pg.group(1))
 
     lines_to_del_to_app = []
-    for (ctx_keys, line) in lines_to_del:
+    for ctx_keys, line in lines_to_del:
         if (
             ctx_keys[0].startswith("router bgp")
             and line
@@ -1043,10 +1064,10 @@ def bgp_delete_move_lines(lines_to_add, lines_to_del):
                     if re_pg:
                         lines_to_del_to_app.append((ctx_keys, line))
 
-    for (ctx_keys, line) in lines_to_del_to_del:
+    for ctx_keys, line in lines_to_del_to_del:
         lines_to_del.remove((ctx_keys, line))
 
-    for (ctx_keys, line) in lines_to_del_to_app:
+    for ctx_keys, line in lines_to_del_to_app:
         lines_to_del.remove((ctx_keys, line))
         lines_to_del.append((ctx_keys, line))
 
@@ -1056,32 +1077,45 @@ def bgp_delete_move_lines(lines_to_add, lines_to_del):
 
 
 def pim_delete_move_lines(lines_to_add, lines_to_del):
-
     # Under interface context, if 'no ip pim' is present
     # remove subsequent 'no ip pim <blah>' options as it
     # they are implicitly deleted by 'no ip pim'.
     # Remove all such depdendent options from delete
     # pending list.
     pim_disable = False
+    lines_to_del_to_del = []
 
-    for (ctx_keys, line) in lines_to_del:
+    index = -1
+    for ctx_keys, line in lines_to_del:
+        index = index + 1
         if ctx_keys[0].startswith("interface") and line and line == "ip pim":
             pim_disable = True
 
+        # no ip msdp peer <> does not accept source so strip it off.
+        if line and line.startswith("ip msdp peer "):
+            pim_msdp_peer = re.search("ip msdp peer (\S+) source (\S+)", line)
+            if pim_msdp_peer:
+                source_sub_str = "source %s" % pim_msdp_peer.group(2)
+                new_line = line.replace(source_sub_str, "").strip()
+                lines_to_del.remove((ctx_keys, line))
+                lines_to_del.insert(index, (ctx_keys, new_line))
+
     if pim_disable:
-        for (ctx_keys, line) in lines_to_del:
+        for ctx_keys, line in lines_to_del:
             if (
                 ctx_keys[0].startswith("interface")
                 and line
-                and line.startswith("ip pim ")
+                and (line.startswith("ip pim ") or line.startswith("ip multicast "))
             ):
-                lines_to_del.remove((ctx_keys, line))
+                lines_to_del_to_del.append((ctx_keys, line))
+
+    for ctx_keys, line in lines_to_del_to_del:
+        lines_to_del.remove((ctx_keys, line))
 
     return (lines_to_add, lines_to_del)
 
 
 def delete_move_lines(lines_to_add, lines_to_del):
-
     lines_to_add, lines_to_del = bgp_delete_move_lines(lines_to_add, lines_to_del)
     lines_to_add, lines_to_del = pim_delete_move_lines(lines_to_add, lines_to_del)
 
@@ -1089,13 +1123,12 @@ def delete_move_lines(lines_to_add, lines_to_del):
 
 
 def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
-
     # Quite possibly the most confusing (while accurate) variable names in history
     lines_to_add_to_del = []
     lines_to_del_to_del = []
 
-    index = 0
-    for (ctx_keys, line) in lines_to_del:
+    index = -1
+    for ctx_keys, line in lines_to_del:
         deleted = False
 
         # no form of route-map description command only
@@ -1110,10 +1143,21 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
             lines_to_del.remove((ctx_keys, line))
             lines_to_del.insert(index, (ctx_keys, "description"))
 
+        # interface x ; description blah
+        # no form of description does not accept any argument,
+        # strip arg before rendering
+        if (
+            ctx_keys[0].startswith("interface ")
+            and line
+            and line.startswith("description ")
+        ):
+            lines_to_del.remove((ctx_keys, line))
+            lines_to_del.insert(index, (ctx_keys, "description"))
+
         # If there is a change in the segment routing block ranges, do it
         # in-place, to avoid requesting spurious label chunks which might fail
         if line and "segment-routing global-block" in line:
-            for (add_key, add_line) in lines_to_add:
+            for add_key, add_line in lines_to_add:
                 if (
                     ctx_keys[0] == add_key[0]
                     and add_line
@@ -1124,7 +1168,6 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
             continue
 
         if ctx_keys[0].startswith("router bgp") and line:
-
             if line.startswith("neighbor "):
                 # BGP changed how it displays swpX peers that are part of peer-group. Older
                 # versions of frr would display these on separate lines:
@@ -1206,7 +1249,7 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
                     bfd_nbr = "neighbor %s" % nbr
                     bfd_search_string = bfd_nbr + r" bfd (\S+) (\S+) (\S+)"
 
-                    for (ctx_keys, add_line) in lines_to_add:
+                    for ctx_keys, add_line in lines_to_add:
                         if ctx_keys[0].startswith("router bgp"):
                             re_add_nbr_bfd_timers = re.search(
                                 bfd_search_string, add_line
@@ -1236,7 +1279,7 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
                     dir = re_nbr_rm.group(3)
                     search = "neighbor%sroute-map(.*)%s" % (neighbor_name, dir)
                     save_line = "EMPTY"
-                    for (ctx_keys_al, add_line) in lines_to_add:
+                    for ctx_keys_al, add_line in lines_to_add:
                         if ctx_keys_al[0].startswith("router bgp"):
                             if add_line:
                                 rm_match = re.search(search, add_line)
@@ -1256,7 +1299,7 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
                                         lines_to_del_to_del.append((ctx_keys_al, line))
 
                     if adjust_for_bgp_node == 1:
-                        for (ctx_keys_dl, dl_line) in lines_to_del:
+                        for ctx_keys_dl, dl_line in lines_to_del:
                             if (
                                 ctx_keys_dl[0].startswith("router bgp")
                                 and len(ctx_keys_dl) > 1
@@ -1432,13 +1475,41 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
                 lines_to_add.append((add_cmd, None))
                 lines_to_del_to_del.append((ctx_keys, None))
 
+        # bgp as-path access-list can be specified without a seq number.
+        # However, the running config always
+        # adds `seq X` (sequence number). So, ignore such lines as well.
+        # Examples:
+        #      bgp as-path access-list important_internet_bgp_as_numbers seq 30 permit _40841_"
+        re_bgp_as_path = re.search(
+            "^(bgp )(as-path )(access-list )(\S+\s+)(seq \d+\s+)(permit|deny)(.*)$",
+            ctx_keys[0],
+        )
+        if re_bgp_as_path:
+            found = False
+            tmpline = (
+                re_bgp_as_path.group(1)
+                + re_bgp_as_path.group(2)
+                + re_bgp_as_path.group(3)
+                + re_bgp_as_path.group(4)
+                + re_bgp_as_path.group(6)
+                + re_bgp_as_path.group(7)
+            )
+            for ctx in lines_to_add:
+                if ctx[0][0] == tmpline:
+                    lines_to_del_to_del.append((ctx_keys, None))
+                    lines_to_add_to_del.append(((tmpline,), None))
+                    found = True
+            if found is False:
+                add_cmd = ("no " + ctx_keys[0],)
+                lines_to_add.append((add_cmd, None))
+                lines_to_del_to_del.append((ctx_keys, None))
+
         if (
             len(ctx_keys) == 3
             and ctx_keys[0].startswith("router bgp")
             and ctx_keys[1] == "address-family l2vpn evpn"
             and ctx_keys[2].startswith("vni")
         ):
-
             re_route_target = (
                 re.search("^route-target import (.*)$", line)
                 if line is not None
@@ -1514,19 +1585,18 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
                         lines_to_del_to_del.append((ctx_keys, line))
                         lines_to_add_to_del.append((tmp_ctx_keys, line))
 
-    for (ctx_keys, line) in lines_to_del_to_del:
+    for ctx_keys, line in lines_to_del_to_del:
         try:
             lines_to_del.remove((ctx_keys, line))
         except ValueError:
             pass
 
-    for (ctx_keys, line) in lines_to_add_to_del:
+    for ctx_keys, line in lines_to_add_to_del:
         try:
             lines_to_add.remove((ctx_keys, line))
         except ValueError:
             pass
 
-
     return (lines_to_add, lines_to_del)
 
 
@@ -1537,8 +1607,7 @@ def ignore_unconfigurable_lines(lines_to_add, lines_to_del):
     """
     lines_to_del_to_del = []
 
-    for (ctx_keys, line) in lines_to_del:
-
+    for ctx_keys, line in lines_to_del:
         # The integrated-vtysh-config one is technically "no"able but if we did
         # so frr-reload would stop working so do not let the user shoot
         # themselves in the foot by removing this.
@@ -1559,7 +1628,7 @@ def ignore_unconfigurable_lines(lines_to_add, lines_to_del):
             log.info('"%s" cannot be removed' % (ctx_keys[-1],))
             lines_to_del_to_del.append((ctx_keys, line))
 
-    for (ctx_keys, line) in lines_to_del_to_del:
+    for ctx_keys, line in lines_to_del_to_del:
         lines_to_del.remove((ctx_keys, line))
 
     return (lines_to_add, lines_to_del)
@@ -1584,8 +1653,7 @@ def compare_context_objects(newconf, running):
 
     # Find contexts that are in newconf but not in running
     # Find contexts that are in running but not in newconf
-    for (running_ctx_keys, running_ctx) in iteritems(running.contexts):
-
+    for running_ctx_keys, running_ctx in iteritems(running.contexts):
         if running_ctx_keys in newconf.contexts:
             newconf_ctx = newconf.contexts[running_ctx_keys]
 
@@ -1606,7 +1674,6 @@ def compare_context_objects(newconf, running):
                         lines_to_del.append((running_ctx_keys, new_del_line))
 
         if running_ctx_keys not in newconf.contexts:
-
             # We check that the len is 1 here so that we only look at ('router bgp 10')
             # and not ('router bgp 10', 'address-family ipv4 unicast'). The
             # latter could cause a false delete_bgpd positive if ipv4 unicast is in
@@ -1777,14 +1844,12 @@ def compare_context_objects(newconf, running):
 
     # Find the lines within each context to add
     # Find the lines within each context to del
-    for (newconf_ctx_keys, newconf_ctx) in iteritems(newconf.contexts):
-
+    for newconf_ctx_keys, newconf_ctx in iteritems(newconf.contexts):
         if newconf_ctx_keys in running.contexts:
             running_ctx = running.contexts[newconf_ctx_keys]
 
             for line in newconf_ctx.lines:
                 if line not in running_ctx.dlines:
-
                     # candidate paths can only be added after the policy and segment list,
                     # so add them to a separate array that is going to be appended at the end
                     if (
@@ -1802,10 +1867,8 @@ def compare_context_objects(newconf, running):
                 if line not in newconf_ctx.dlines:
                     lines_to_del.append((newconf_ctx_keys, line))
 
-    for (newconf_ctx_keys, newconf_ctx) in iteritems(newconf.contexts):
-
+    for newconf_ctx_keys, newconf_ctx in iteritems(newconf.contexts):
         if newconf_ctx_keys not in running.contexts:
-
             # candidate paths can only be added after the policy and segment list,
             # so add them to a separate array that is going to be appended at the end
             if (
@@ -2040,7 +2103,6 @@ if __name__ == "__main__":
         reload_ok = False
 
     if args.test:
-
         # Create a Config object from the running config
         running = Config(vtysh)
 
@@ -2056,8 +2118,7 @@ if __name__ == "__main__":
                 print("\nLines To Delete")
                 print("===============")
 
-            for (ctx_keys, line) in lines_to_del:
-
+            for ctx_keys, line in lines_to_del:
                 if line == "!":
                     continue
 
@@ -2083,8 +2144,7 @@ if __name__ == "__main__":
                 print("\nLines To Add")
                 print("============")
 
-            for (ctx_keys, line) in lines_to_add:
-
+            for ctx_keys, line in lines_to_add:
                 if line == "!":
                     continue
 
@@ -2172,8 +2232,7 @@ if __name__ == "__main__":
             # apply to other scenarios as well where configuring FOO adds BAR
             # to the config.
             if lines_to_del and x == 0:
-                for (ctx_keys, line) in lines_to_del:
-
+                for ctx_keys, line in lines_to_del:
                     if line == "!":
                         continue
 
@@ -2203,12 +2262,11 @@ if __name__ == "__main__":
                             vtysh(["configure"] + cmd, stdouts)
 
                         except VtyshException:
-
                             # - Pull the last entry from cmd (this would be
                             #   'no ip ospf authentication message-digest 1.1.1.1' in
                             #   our example above
                             # - Split that last entry by whitespace and drop the last word
-                            log.info("Failed to execute %s", " ".join(cmd))
+                            log.error("Failed to execute %s", " ".join(cmd))
                             last_arg = cmd[-1].split(" ")
 
                             if len(last_arg) <= 2:
@@ -2231,8 +2289,7 @@ if __name__ == "__main__":
             if lines_to_add:
                 lines_to_configure = []
 
-                for (ctx_keys, line) in lines_to_add:
-
+                for ctx_keys, line in lines_to_add:
                     if line == "!":
                         continue
 
