@@ -5,6 +5,9 @@
 
 #include <zebra.h>
 
+#include <fcntl.h>
+#include <signal.h>
+#include <sys/stat.h>
 #include <sys/un.h>
 #include <setjmp.h>
 #include <pwd.h>
@@ -362,8 +365,7 @@ int main(int argc, char **argv, char **env)
 
 	strlcpy(sysconfdir, frr_sysconfdir, sizeof(sysconfdir));
 
-	frr_init_vtydir();
-	strlcpy(vtydir, frr_vtydir, sizeof(vtydir));
+	strlcpy(vtydir, frr_runstatedir, sizeof(vtydir));
 
 	/* Option handling. */
 	while (1) {
